<?php

namespace FiloBlu\Flow\Model\Actions\From;

class Addtocategory {

    protected $_namesPaths = null;

    CONST ENTITY_TYPE_CODE = 'catalog_product';

    /**
     * Categories text-path to ID hash.
     *
     * @var array
     */
    protected $_categories = array();
    /**
     * Categories text-path to ID hash with roots checking.
     *
     * @var array
     */
    protected $_categoriesWithRoots = array();


    public function __construct(
        \Magento\Framework\Model\Context $context,
        \Magento\Framework\Registry $registry,
        \FiloBlu\Flow\Helper\Product $helper_product,
        \Magento\Framework\Model\ResourceModel\AbstractResource $resource = null,
        \Magento\Framework\Data\Collection\AbstractDb $resourceCollection = null,
        array $data = []
    ) {
        $this->helper_product = $helper_product;
        $this->_initCategories();
    }

    public function process($product, $value, $storeId) {

        if (empty($value)) {
            return true;
        }

        if(empty($this->_categories) || empty($this->_categories)){
            $this->_initCategories();
        }


        $position = 0;
        $originalValue = $value;
        $values = explode('#', $value);
        if (count($values) == 2) {
            $value = $values[0];
            $position = $values[1];
        }
        else {
            $value = $values[0];
        }

        $currentCategoryIds = $product->getCategoryIds();
        // position must be set even if product already exists (EG: positions update)
        # if(!in_array($value, $currentCategoryIds)){

        if (is_numeric($value)) {
            $category = \Magento\Framework\App\ObjectManager::getInstance()->create('Magento\Catalog\Model\Category')->load($value);
//                Mage::getModel('catalog/category')->load($value);

        } else {
            $category = \Magento\Framework\App\ObjectManager::getInstance()->create('\Magento\Catalog\Model\ResourceModel\Category\Collection')
//                Mage::getModel('catalog/category')->getCollection()
                ->addAttributeToFilter('path', array('like' => $this->getCategoryIdsPathByNamesPath($value, false, true)))
                ->getFirstItem();
            if (is_object($category) && $category->getId() > 0) {
                $category = $category->load($category->getId());
            }

            if(is_numeric($catId = $this->getIdFromPath($value))){
                $category = \Magento\Framework\App\ObjectManager::getInstance()->create('Magento\Catalog\Model\Category')->load($catId);
            }
        }

        if  (!is_object($category) || $category->getId() <= 0) {
            return "Category ID {$value} does not exist";
        }

        $old_categories = $product->getCategoryIds();

        $path = $category->getPath();
        $category_ids = explode('/', $path);

        if (count($old_categories))
            $merged_category = array_merge($old_categories,$category_ids);
        else
            $merged_category = $category_ids;

        $merged_category = array_unique($merged_category);
        $merged_category = array_diff($merged_category, array(1, 2/*Mage::app()->getDefaultStoreView()->getRootCategoryId()*/));

        if (true/*(bool)Mage::getStoreConfig('fbflow/new_options_status/configurable_category')*/) {

            if ($product->getData('visibility') != '1') {

                $product->setCategoryIds($merged_category);
                $product->save();

            }

            else {

                $product->setCategoryIds(array());
                $product->save();

            }

        }

        else {

            $product->setCategoryIds($merged_category);
            $product->save();

        }


        if ($product->getTypeId() == \Magento\Catalog\Model\Product\Type::DEFAULT_TYPE) {
            $parentIds = \Magento\Framework\App\ObjectManager::getInstance()->get('Magento\ConfigurableProduct\Model\ResourceModel\Product\Type\Configurable')
            //Mage::getResourceSingleton('catalog/product_type_configurable')
                ->getParentIdsByChild($product->getId());
            if (count($parentIds) <= 0)
                return true;

            foreach ($parentIds as $parentId) {
                $parent_product = \Magento\Framework\App\ObjectManager::getInstance()->create('Magento\Catalog\Model\Product')->load($parentId);
                //Mage::getModel('catalog/product')->load($parentId);
                $merged_parent_category = array_unique(array_merge($merged_category,$parent_product->getCategoryIds()));
                $parent_product->setCategoryIds($merged_parent_category)->save();
            }

        }

//        $positions = $category->getProductsPosition();
//
//        if (preg_match("/#/",$originalValue) || !isset($positions[$product->getId()]))
//            $positions[$product->getId()] = $position;
//
//        $category->setPostedProducts($positions);
//        $category->save();
        #}

        return true;
    }

    /**
     * Populates the models properties with category information.
     */
//    public function __construct()
//    {
//        $this->_initCategories();
//    }
    /**
     * Finds a subcategory id from a path string
     *
     * @param $string
     * @return bool
     */
    public function getIdFromPath($string)
    {
        if (in_array(strtoupper($string), array_keys($this->_categories))) {
            return $this->_categories[strtoupper($string)];
        }
        return false;
    }
    /**
     * Returns all valid path strings
     *
     * @return array
     */
    public function getAllPaths()
    {
        return array_keys($this->_categories);
    }
    /**
     * Initialize categories text-path to ID hash.
     *
     * @return Mage_ImportExport_Model_Import_Entity_Product
     */
    protected function _initCategories()
    {
        $collection = \Magento\Framework\App\ObjectManager::getInstance()->create('\Magento\Catalog\Model\ResourceModel\Category\Collection');
        $collection->setStore(0);
        $collection->addNameToResult();
//        $collection = Mage::getResourceModel('catalog/category_collection')->setStore(0)->addNameToResult();
        /* @var $collection Mage_Catalog_Model_Resource_Eav_Mysql4_Category_Collection */
        foreach ($collection as $category) {
            $structure = explode('/', $category->getPath());
            $pathSize = count($structure);
            if ($pathSize > 1) {
                $path = array();
                for ($i = 1; $i < $pathSize; $i++) {
                    $path[] = $collection->getItemById($structure[$i])->getName();
                }
                $rootCategoryName = array_shift($path);
                if (!isset($this->_categoriesWithRoots[$rootCategoryName])) {
                    $this->_categoriesWithRoots[$rootCategoryName] = array();
                }
                $index = implode('/', $path);
                $index = strtoupper($index);
                $this->_categoriesWithRoots[$rootCategoryName][$index] = $category->getId();
                if ($pathSize > 2) {
                    $this->_categories[$index] = $category->getId();
                }
            }
        }
        return $this;
    }


    protected function getCategoryIdsPathByNamesPath($names_path, $includeRoot = true, $addLike = false) {
        $paths = $this->getCategoryNamesPaths();
        if (!isset($paths['name2id']) || count($paths['name2id']) <= 0) {
            return false;
        }

        $name2id = $paths['name2id'];

        $pathParts = explode('/', $names_path);
        if (!is_array($pathParts) || count($pathParts) <= 0) {
            return false;
        }

        $ids_path_result = array();
        foreach ($pathParts as $singlePathName) {
            if (!isset($name2id[trim($singlePathName)])) {
                return false;
            }

            $ids_path_result[] = $name2id[$singlePathName];
        }

        $ids_path = implode('/', $ids_path_result);
        if ($includeRoot) {
            $ids_path = Mage::app()->getDefaultStoreView()->getRootCategoryId() . '/' . $ids_path;
        }

        if ($addLike) {
            $ids_path = '%' . $ids_path;
        }
        return $ids_path;
    }

    protected function getCategoryNamesPaths() {
        if (is_null($this->_namesPaths)) {
            $this->_namesPaths = array('id2name' => array(),'name2id' => array());
            $categoriesCollection = \Magento\Framework\App\ObjectManager::getInstance()->create('\Magento\Catalog\Model\ResourceModel\Category\Collection')
                ->setStore(0)
                ->addAttributeToSelect('*');

            foreach ($categoriesCollection as $singleCategory) {
                $categoryName = trim($singleCategory->getName());

                $this->_namesPaths['id2name'][$singleCategory->getId()] = $categoryName;
                $this->_namesPaths['name2id'][$categoryName] = $singleCategory->getId();
            }
        }

        return $this->_namesPaths;
    }

}